package com.onsiteservice.common.annotation.sql;

import com.google.common.base.CaseFormat;
import com.onsiteservice.util.ReflectUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.BeanUtils;
import org.springframework.cglib.beans.BeanGenerator;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.LinkedList;
import java.util.List;

@Slf4j
@Aspect
@Component
public class AutoConditionAspect {

    private static final String CONDITIONS = "conditions";

    @Pointcut("@annotation(com.onsiteservice.common.annotation.sql.AutoCondition)")
    public void autoCondition() {
    }

    @Around("autoCondition()")
    public Object around(ProceedingJoinPoint joinPoint) throws Throwable {
        var args = joinPoint.getArgs();
        appendConditions(args);
        Object result = joinPoint.proceed(args);
        return result;
    }

    /**
     * 添加条件
     */
    private void appendConditions(Object[] args) {
        if (args == null || args.length == 0) {
            return;
        }
        var arg = args[0];
        // 先获取字段中的注解值
        var conditions = extractConditionInfo(arg);
        // 将条件集合放回对象
        if (arg instanceof AutoSqlCondition) {
            // 如果实现了AutoSqlCondition接口则直接强转赋值
            ((AutoSqlCondition) arg).setConditions(conditions);
        } else {
            // 如果没有实现AutoSqlCondition接口，则使用cglib添加一个名为conditions的属性
            var newArg = addConditionsToObj(arg);
            BeanUtils.copyProperties(arg, newArg);
            ReflectUtils.invokeSetter(newArg, CONDITIONS, conditions);
            // 替换原参数
            args[0] = newArg;
        }
    }

    private Object addConditionsToObj(Object obj) {
        BeanGenerator beanGenerator = new BeanGenerator();
        beanGenerator.setSuperclass(obj.getClass());
        beanGenerator.setUseCache(true);
        beanGenerator.addProperty(CONDITIONS, List.class);
        return beanGenerator.create();
    }

    /**
     * 从对象中获取条件值
     */
    private List<ConditionInfo> extractConditionInfo(Object arg) {
        var filedList = arg.getClass().getDeclaredFields();
        var conditions = new LinkedList<ConditionInfo>();
        // 获取类型注解
        var typeAnno = arg.getClass().getDeclaredAnnotation(FieldCondition.class);
        String defaultAlias = null;
        if(typeAnno != null) {
            defaultAlias = typeAnno.alias();
        }
        for (Field field : filedList) {
            var anno = field.getDeclaredAnnotation(FieldCondition.class);
            Object value = ReflectUtils.invokeGetter(arg, field.getName());
            if (value != null && anno != null) {
                // 列名
                String prefix;
                if(StringUtils.isEmpty(anno.alias())) {
                    if(StringUtils.isEmpty(defaultAlias)) {
                        prefix = "";
                    } else {
                        prefix = defaultAlias + ".";
                    }
                } else {
                    prefix = anno.alias() + ".";
                }
                var conditionInfo = new ConditionInfo();
                conditionInfo.setValue(value);
                if (StringUtils.isEmpty(anno.column())) {
                    // 没有指定列名，使用字段名称转下划线
                    conditionInfo.setColumn(prefix + CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, field.getName()));
                } else {
                    conditionInfo.setColumn(prefix + anno.column());
                }
                // 比较方式
                if (anno.rel().equals(ConditionEnum.NONE)) {
                    // 没有指定比较方式，根据类型自动判断
                    if (value instanceof String) {
                        conditionInfo.setRel(ConditionEnum.LIKE.getOperator());
                    } else {
                        conditionInfo.setRel(ConditionEnum.EQ.getOperator());
                    }
                } else {
                    conditionInfo.setRel(anno.rel().getOperator());
                }
                conditions.add(conditionInfo);
            }
        }
        return conditions;
    }


}
